let accessToken = ''
let conversationPage = 0
let conversation = []

const requestAccessToken = async () => {
  const response = await fetch('https://chat.openai.com/api/auth/session', {
    headers: {
      accept: '*/*',
      'accept-language': 'en-US,en;q=0.9,es;q=0.8',
      'cache-control': 'no-cache',
      pragma: 'no-cache',
      'sec-fetch-dest': 'empty',
      'sec-fetch-mode': 'cors',
      'sec-fetch-site': 'same-origin',
    },
    referrer: 'https://chat.openai.com/',
    referrerPolicy: 'same-origin',
    body: null,
    method: 'GET',
    mode: 'cors',
    credentials: 'include',
  })
  const data = await response.json()

  const { accessToken } = data

  return accessToken
}

const requestPaginatedConversation = async () => {
  const limitPerPage = 100
  const offset = conversationPage * limitPerPage
  const response = await fetch(
    `https://chat.openai.com/backend-api/conversations?offset=${offset}&limit=${limitPerPage}&order=updated`,
    {
      headers: {
        accept: '*/*',
        'accept-language': 'en-US',
        authorization: `Bearer ${accessToken}`,
        'cache-control': 'no-cache',
        pragma: 'no-cache',
      },
      referrer: 'https://chat.openai.com/',
      referrerPolicy: 'same-origin',
      body: null,
      method: 'GET',
      mode: 'cors',
      credentials: 'include',
    },
  )
  const data = await response.json()

  return data
}

const fetchAllConversation = async () => {
  const data = await requestPaginatedConversation()
  const { total, items } = data
  conversation = [...conversation, ...items]
  while (conversation.length < total) {
    conversationPage += 1
    const data = await requestPaginatedConversation()
    const { items } = data
    conversation = [...conversation, ...items]
  }
}

const start = async () => {
  accessToken = await requestAccessToken()
  await fetchAllConversation()
  setTimeout(() => {
    addSearchBar()
    addSearchResultsWrapper()
    markConversationWithTags()
  }, 200)
}

const addTagToConversation = (conversationId) => {
  const storage = JSON.parse(localStorage.getItem('tags') || `{}`)
  let div = document.getElementById(`tag-${conversationId}`)
  if (!div) {
    div = document.createElement('div')
    div.id = `tag-${conversationId}`
    div.style.marginLeft = '12px'
    div.style.marginTop = '12px'
    div.style.display = 'flex'
    div.style.flexDirection = 'row'
    div.style.overflowX = 'auto'
    document.getElementById(`item-${conversationId}`).prepend(div)
  }
  div.innerHTML = ''
  const button = document.createElement('button')
  button.innerHTML = button.innerHTML =
    '<svg xmlns="http://www.w3.org/2000/svg" width="18" height="18" viewBox="0 0 24 24" fill="none"><path d="m4.405 15.526 4.53 4.53a4.78 4.78 0 0 0 6.75 0l4.39-4.39a4.78 4.78 0 0 0 0-6.75l-4.54-4.52a4.75 4.75 0 0 0-3.6-1.39l-5 .24c-2 .09-3.59 1.68-3.69 3.67l-.24 5c-.06 1.35.45 2.66 1.4 3.61Z" stroke="#CCC" stroke-width="1.5" stroke-linecap="round" stroke-linejoin="round"></path><path d="M9.735 12.226a2.5 2.5 0 1 0 0-5 2.5 2.5 0 0 0 0 5Z" stroke="#CCC" stroke-width="1.5" stroke-linecap="round"></path><path d="m13.235 17.226 4-4" stroke="#CCC" stroke-width="1.5" stroke-miterlimit="10" stroke-linecap="round" stroke-linejoin="round"></path></svg>'
  button.onclick = (e) => {
    e.stopPropagation()
    openTagForm(conversationId)
  }
  div.appendChild(button)
  const id = conversationId.replace('$', '')
  const tag = storage[id] || '[untagged]'
  if (tag) {
    tag.split(' ').forEach((tagItem) => {
      const tagElementDiv = document.createElement('div')
      tagElementDiv.innerText = `#${tagItem}`
      tagElementDiv.style.color = '#fff'
      tagElementDiv.style.marginLeft = '4px'
      tagElementDiv.style.display = 'flex'
      tagElementDiv.style.background = '#333'
      tagElementDiv.style.fontSize = '11px'
      tagElementDiv.style.borderRadius = '10px'
      tagElementDiv.style.paddingLeft = '8px'
      tagElementDiv.style.paddingRight = '8px'
      tagElementDiv.style.justifyContent = 'center'
      tagElementDiv.style.marginBottom = '2px'
      tagElementDiv.style.whiteSpace = 'nowrap'
      if (tagItem === '[untagged]') {
        tagElementDiv.style.fontStyle = 'italic'
        tagElementDiv.style.color = '#818188'
      }
      if (tagItem !== '[untagged]') {
        tagElementDiv.style.cursor = 'pointer'
        tagElementDiv.onclick = (e) => {
          e.stopPropagation()
          addToSearchBar(tagItem)
        }
      }
      div.appendChild(tagElementDiv)
    })
  }
}

const addToSearchBar = (term) => {
  const textInput = document.getElementById('search-bar-input')
  if (!textInput) {
    return
  }

  textInput.value = `#${term}`
  search(`#${term}`)
}

const markConversationWithTags = () => {
  const list = document.querySelectorAll('[aria-label="Chat history"] li')
  list.forEach((li) => {
    const text = li.innerText
    const conversationItem = conversation.find((item) => item.title === text)
    if (conversationItem) {
      li.id = `item-${conversationItem.id}`
      addTagToConversation(conversationItem.id)
    }
  })
}

const removeOldForms = () => {
  ;[...document.getElementsByClassName('tag-form')].forEach((form) => {
    form.remove()
  })
}

const openTagForm = (conversationId) => {
  removeOldForms()

  const storage = JSON.parse(localStorage.getItem('tags') || `{}`)
  const id = conversationId.replace('$', '')
  const value = storage[id] || ''

  const element = document.getElementById(`item-${conversationId}`)
  const div = document.createElement('div')
  div.id = `tag-form-${conversationId}`
  div.classList.add('tag-form')
  div.style.position = 'absolute'
  div.style.zIndex = '9999'
  div.style.backgroundColor = 'rgb(52 53 65)'
  div.style.marginLeft = '10px'
  div.style.paddingLeft = '5px'
  div.style.paddingRight = '5px'
  div.style.borderRadius = '5px'
  div.style.padding = '5px'
  div.style.display = 'flex'
  div.style.alignContent = 'center'
  div.style.justifyContent = 'center'
  div.style.alignItems = 'center'

  const inputText = document.createElement('input')
  inputText.id = `tag-form-text-${conversationId}`
  inputText.style.outline = 'none'
  inputText.style.padding = '2px'
  inputText.style.borderRadius = '4px'
  inputText.style.fontSize = '12px'
  inputText.style.color = '#333'
  inputText.value = value
  inputText.addEventListener('keyup', function (event) {
    if (event.key === 'Enter') {
      saveTagConversation(conversationId)
      addTagToConversation(conversationId)
      addTagToConversation(conversationId.replace('$', ''))
      removeOldForms()
    }
  })

  const buttonSave = document.createElement('button')
  buttonSave.innerHTML =
    '<?xml version="1.0" ?><svg fill="none" height="24" stroke="white" stroke-linecap="round" stroke-linejoin="round" stroke-width="2" viewBox="0 0 24 24" width="24" xmlns="http://www.w3.org/2000/svg"><polyline points="20 6 9 17 4 12"/></svg>'
  buttonSave.style.width = '20px'
  buttonSave.style.height = '20px'
  buttonSave.style.marginLeft = '4px'
  buttonSave.style.marginRight = '4px'
  buttonSave.onclick = () => {
    saveTagConversation(conversationId)
    addTagToConversation(conversationId)
    addTagToConversation(conversationId.replace('$', ''))
    removeOldForms()
  }
  const buttonCancel = document.createElement('button')
  buttonCancel.innerHTML =
    '<?xml version="1.0" ?><svg viewBox="0 0 32 32" xmlns="http://www.w3.org/2000/svg"><defs><style>.cls-1{height:20px; width:20px; fill:none;stroke:#FFF;stroke-linecap:round;stroke-linejoin:round;stroke-width:2px;}</style></defs><title/><g id="cross"><line class="cls-1" x1="7" x2="25" y1="7" y2="25"/><line class="cls-1" x1="7" x2="25" y1="25" y2="7"/></g></svg>'
  buttonCancel.style.width = '20px'
  buttonCancel.style.height = '20px'
  buttonCancel.style.marginLeft = '4px'
  buttonCancel.style.marginRight = '4px'
  buttonCancel.onclick = () => {
    removeOldForms()
  }

  div.appendChild(inputText)
  div.appendChild(buttonSave)
  div.appendChild(buttonCancel)
  setTimeout(() => {
    inputText.focus()
  }, 500)

  element.prepend(div)
}

const saveTagConversation = (conversationId) => {
  const textField = document.getElementById(`tag-form-text-${conversationId}`)
  const storage = JSON.parse(localStorage.getItem('tags') || `{}`)
  const id = conversationId.replace('$', '')
  localStorage.setItem(
    'tags',
    JSON.stringify({
      ...storage,
      ...{ [`${id}`]: textField.value },
    }),
  )
}

const setNavBarWrapper = () => {
  const elements = document.querySelectorAll('*[style]')
  ;[...elements].forEach((element) => {
    if (element.style.width === '260px') {
      element.style.width = '300px'
      return element
    }
  })
  document.querySelector(
    '.flex-col.flex-1.transition-opacity.duration-500.-mr-2.pr-2.overflow-y-auto',
  ).style.width = '300px'
}

const addSearchResultsWrapper = () => {
  const searchResultsWrapper = document.querySelector(
    '.flex.flex-col.gap-2.pb-2.text-gray-800.text-sm div span',
  )

  const searchResults = document.createElement('div')
  searchResults.id = 'search-results'
  searchResults.style.width = '100%'
  searchResults.style.height = '100%'
  searchResultsWrapper.prepend(searchResults)
}

const addSearchBar = () => {
  const searchBar = document.createElement('div')
  searchBar.style.width = '100%'
  searchBar.style.height = '40px'
  searchBar.style.backgroundColor = '#333'
  searchBar.style.display = 'flex'
  searchBar.style.flexDirection = 'row'
  searchBar.style.alignItems = 'center'
  searchBar.style.paddingLeft = '10px'
  searchBar.style.paddingRight = '10px'
  searchBar.style.marginBottom = '10px'
  searchBar.style.borderRadius = '5px'

  const inputText = document.createElement('input')
  inputText.id = 'search-bar-input'
  inputText.style.width = '100%'
  inputText.style.height = '100%'
  inputText.style.backgroundColor = '#333'
  inputText.style.border = 'none'
  inputText.style.color = '#fff'
  inputText.style.fontSize = '14px'
  inputText.style.outline = 'none'
  inputText.style.paddingLeft = '10px'
  inputText.style.paddingRight = '10px'
  inputText.style.borderRadius = '5px'
  inputText.placeholder = 'Search Chat...'
  inputText.onkeyup = (e) => {
    const term = e.target.value
    if (!term || term.length < 2) {
      return addSearchResults(null)
    }
    search(term.toLowerCase())
  }
  searchBar.appendChild(inputText)

  const wrapper = document.querySelector('.mb-1.flex.flex-row.gap-2')
  wrapper.parentNode.insertBefore(searchBar, wrapper.nextSibling)
}

const addSearchResults = (results, term) => {
  const resultsWrapper = document.getElementById('search-results')
  resultsWrapper.innerHTML = ''
  console.log(results.length)
  if (!results && !term) {
    return
  }
  if (!results.length && term) {
    const listDiv = document.createElement('div')
    listDiv.innerHTML = `<div class="h-9 pb-2 pt-3 px-3 text-xs text-gray-500 font-medium text-ellipsis overflow-hidden break-all bg-gray-50 gizmo:bg-white dark:bg-gray-900" >
      Nothing found for "${term}"
    </div>`
    resultsWrapper.appendChild(listDiv)
    return
  }
  results.forEach((item) => {
    const listDiv = document.createElement('div')

    listDiv.innerHTML = `
    <div>
    <div id="item-$${item.id}"></div>
    <a
    class="flex py-3 px-3 items-center gap-3 relative rounded-md hover:bg-gray-100 dark:hover:bg-[#2A2B32] cursor-pointer break-all bg-gray-50 hover:pr-4 dark:bg-gray-900 group"
    href="/c/${item.id}"
    >
      <svg
        stroke="currentColor"
        fill="none"
        stroke-width="2"
        viewBox="0 0 24 24"
        stroke-linecap="round"
        stroke-linejoin="round"
        class="icon-sm"
        height="1em"
        width="1em"
        xmlns="http://www.w3.org/2000/svg"
      >
        <path
          d="M21 15a2 2 0 0 1-2 2H7l-4 4V5a2 2 0 0 1 2-2h14a2 2 0 0 1 2 2z"
        ></path>
      </svg>
      <div class="flex-1 text-ellipsis max-h-5 overflow-hidden break-all relative">
        ${item.title}
        <div
          class="absolute inset-y-0 right-0 w-8 z-10 bg-gradient-to-l dark:from-gray-900 from-gray-50 gizmo:from-white group-hover:from-gray-100 dark:group-hover:from-[#2A2B32]"
        ></div>
      </div>
  </a></div>`
    resultsWrapper.appendChild(listDiv)
    addTagToConversation(`$${item.id}`)
  })
}

const search = (term) => {
  if (term.startsWith('#')) {
    const storage = JSON.parse(localStorage.getItem('tags') || `{}`)
    const similarities = {}
    Object.keys(storage).forEach((key) => {
      similarities[key] = `${storage[key]}`.includes(term.replace('#', ''))
        ? 1
        : 0
    })
    const results = conversation
      .map((item) => {
        const similarity = similarities[item.id] || 0
        return {
          ...item,
          similarity,
        }
      })
      .filter((s) => s.similarity >= 0.1)
      .sort((a, b) => b.similarity - a.similarity)

    return addSearchResults(results, term)
  }
  const results = conversation
    .map((item) => {
      const title = item.title.toLowerCase()
      const similarity = stringSimilarity(term, title)
      return {
        ...item,
        similarity,
      }
    })
    .filter((s) => s.similarity >= 0.1)
    .sort((a, b) => b.similarity - a.similarity)
  addSearchResults(results, term)
}

const stringSimilarity = (pattern, compare, gramSize = 2) => {
  function getNGrams(s, len) {
    s = ' '.repeat(len - 1) + s.toLowerCase() + ' '.repeat(len - 1)
    let v = new Array(s.length - len + 1)
    for (let i = 0; i < v.length; i++) {
      v[i] = s.slice(i, i + len)
    }
    return v
  }
  if (
    !(pattern === null || pattern === void 0 ? void 0 : pattern.length) ||
    !(compare === null || compare === void 0 ? void 0 : compare.length)
  ) {
    return 0.0
  }
  let s1 = pattern.length < compare.length ? pattern : compare
  let s2 = pattern.length < compare.length ? compare : pattern
  let pairs1 = getNGrams(s1, gramSize)
  let pairs2 = getNGrams(s2, gramSize)
  let set = new Set(pairs1)
  let total = pairs2.length
  let hits = 0
  for (let item of pairs2) {
    if (set.delete(item)) {
      hits++
    }
  }
  return hits / total
}

start()
